import React, { useEffect } from "react";
import { useParams } from "react-router-dom";
import { VncScreen } from "react-vnc";
import { ServerApi } from "../api/ServerApi";
import { VMApi, VMInfo } from "../api/VMApi";
import { useSnackbar } from "../hooks/providers/SnackbarProvider";
import { time } from "../utils/DateUtils";
import { AsyncWidget } from "../widgets/AsyncWidget";

interface VNCTokenInfo {
  url: string;
  expire: number;
}

export function VNCRoute(): React.ReactElement {
  const { uuid } = useParams();

  const [vm, setVM] = React.useState<VMInfo | undefined>();

  const load = async () => {
    setVM(await VMApi.GetSingle(uuid!));
  };

  return (
    <AsyncWidget
      loadKey={uuid}
      load={load}
      errMsg="Failed to load VM information!"
      build={() => <VNCInner vm={vm!} />}
    />
  );
}

function VNCInner(p: { vm: VMInfo }): React.ReactElement {
  const snackbar = useSnackbar();

  const [token, setToken] = React.useState<VNCTokenInfo | undefined>();
  const [counter, setCounter] = React.useState(1);

  const connect = async (force: boolean) => {
    try {
      if (force) setCounter(counter + 1);

      // Check if getting new time is useless
      if ((token?.expire ?? 0) > time()) return;

      setToken(undefined);

      const u = await VMApi.OneShotVNCURL(p.vm);

      if (!token)
        setToken({
          expire: time() + ServerApi.Config.constraints.vnc_token_duration,
          url: u,
        });
    } catch (e) {
      console.error(e);
      snackbar("Failed to initialize VNC connection!");
    }
  };

  const disconnected = () => {
    connect(true);
  };

  useEffect(() => {
    connect(false);
  });

  if (token === undefined)
    return <p>Please wait, connecting to the machine...</p>;

  return (
    <div
      style={{
        display: "flex",
        alignItems: "center",
        justifyContent: "center",
        height: "100%",
      }}
    >
      <VncScreen
        url={token.url}
        onDisconnect={() => {
          console.info("VNC disconnected " + token?.url);
          disconnected();
        }}
      />
    </div>
  );
}
